"""
Concordia Class Search command line tool.
"""

import argparse
import logging

from . import api
from . import utils


def _list(args):
    if args.terms:
        terms = api.terms()
        utils.display_terms(terms)
    if args.careers:
        careers = api.careers()
        utils.display_careers(careers)
    if args.subjects:
        subjects = api.subjects()
        utils.display_subjects(subjects)


def _search(args):
    result = api.class_search(args.term, args.career, args.subject, args.number, args.details)
    if args.dir:
        utils.save_class_search(result, args.dir)
    utils.display_class_search(result)


def main():
    parser = argparse.ArgumentParser(description='Search and view Concordia University classes.')

    logging_group = parser.add_argument_group('logging level').add_mutually_exclusive_group()
    logging_group.add_argument('-v', '--verbose', action='store_const',
                               const=logging.DEBUG, default=logging.CRITICAL,
                               dest='loglevel', help='enable verbose output')
    logging_group.add_argument('-d', '--debug', action='store_const',
                               const=logging.INFO, dest='loglevel',
                               help='show debugging statements')

    subparsers = parser.add_subparsers()

    list_parser = subparsers.add_parser('list')
    list_group = list_parser.add_mutually_exclusive_group()
    list_group.add_argument('-t', '--terms', action='store_true')
    list_group.add_argument('-c', '--careers', action='store_true')
    list_group.add_argument('-s', '--subjects', action='store_true')
    list_parser.set_defaults(func=_list)

    search_parser = subparsers.add_parser('search')
    search_parser.add_argument('-t', '--term', required=True, help='the term value (example: 2171)')
    search_parser.add_argument('-c', '--career', required=True, help='the career value (example: ugrd)')
    search_parser.add_argument('-s', '--subject', required=True, help='the subject value (example: comp)')
    search_parser.add_argument('-n', '--number', required=True, help='the class number (example: 352)')
    search_parser.add_argument('--details', action='store_true',
                               help='get section details (will not be displayed)')
    search_parser.add_argument('--dir', nargs='?', const='data', default=None,
                               help='save results to files under DIR (default: data)')
    search_parser.add_argument('--existing', action='store_true',
                               help='do not overwrite existing results')
    search_parser.set_defaults(func=_search)

    args = parser.parse_args()
    logging.basicConfig(level=args.loglevel)
    args.func(args)
