"""
Module containing concordia_class_search utilities.
"""

import collections
import copy
import json
import os

import tabulate


class_attributes = ['subject', 'number', 'name', 'sections']


section_attributes = ['number', 'section', 'daytime', 'room', 'instructor',
                      'dates', 'term', 'status']


section_detail_attributes = ['status', 'class_number', 'session', 'units',
                             'instruction_mode', 'class_components', 'career',
                             'dates', 'grading', 'location', 'campus',
                             'class_capacity', 'enrollment_total',
                             'available_seats', 'wait_list_capacity',
                             'wait_list_total', 'class_notes', 'description',
                             'textbook']


def tabulate_results(results, column_order, row_order, reverse=False):
    l = sorted(results, key=lambda x: x[row_order], reverse=reverse)
    l = [[(k,v) for k,v in item.items() if k in column_order] for item in l]
    l = [collections.OrderedDict(sorted(item, key=lambda i: column_order.index(i[0]))) for item in l]
    return tabulate.tabulate(l, headers='keys')


def display_terms(terms):
    terms = copy.deepcopy(terms)
    print(tabulate_results(terms, ['value', 'text'], 'value'))


def display_careers(careers):
    careers = copy.deepcopy(careers)
    print(tabulate_results(careers, ['value', 'text'], 'value'))


def display_subjects(subjects):
    subjects = copy.deepcopy(subjects)
    print(tabulate_results(subjects, ['value', 'text'], 'value'))


def display_class_search(class_search):
    class_search = copy.deepcopy(class_search)
    print('{}-{}-{}-{} {}'.format(
        class_search['class_search']['term']['value'],
        class_search['class_search']['career']['value'],
        class_search['class_search']['subject']['value'],
        class_search['class_search']['number'],
        class_search['class_search']['datetime'],
    ))
    for class_ in class_search['classes']:
        print('\n{} {} - {}'.format(
            class_['subject']['value'],
            class_['number'],
            class_['name'],
        ))
        for section in class_['sections']:
            section['term'] = section['term']['text']
        print(tabulate_results(class_['sections'], section_attributes, 'number'))


def save_class_search(class_search, dir_name):
    path = '{}/{}/{}'.format(
        dir_name,
        class_search['class_search']['term']['value'],
        class_search['class_search']['career']['value'],
    ).lower()

    if class_search['class_search']['number']:
        path = '{}/{}'.format(path, class_search['class_search']['subject']['value']).lower()
        filename = class_search['class_search']['number']
    else:
        filename = class_search['class_search']['subject']['value'].lower()

    os.makedirs(path, exist_ok=True)
    with open(f'{path}/{filename}.json', 'w') as f:
        json.dump(class_search, f, indent=2, sort_keys=True)
